unit Int64Em;

{
  Inno Setup
  Copyright (C) 1997-2004 Jordan Russell
  Portions by Martijn Laan
  For conditions of distribution and use, see LICENSE.TXT.

  Declaration of the Integer64 type - which represents an *unsigned* 64-bit
  integer value - and functions for manipulating Integer64's.
  (We can't use the Int64 type since it's only available in Delphi 4 and
  later.)

  $jrsoftware: issrc/Projects/Int64Em.pas,v 1.9 2004/11/07 19:23:06 jr Exp $

  Modified for InnoUnp: make use of UInt64.
}

interface

{$I VERSION.INC}

type
  Integer64 = packed record
    Lo, Hi: Longword;
  end;

function Compare64(const N1, N2: Integer64): Integer;
procedure Dec64(var X: Integer64; N: LongWord);
procedure Dec6464(var X: Integer64; const N: Integer64);
function Div64(var X: Integer64; const Divisor: LongWord): LongWord;
procedure Inc64(var X: Integer64; N: LongWord);
procedure Inc6464(var X: Integer64; const N: Integer64);
function Integer64ToStr(X: Integer64): String;
function Mod64(const X: Integer64; const Divisor: LongWord): LongWord;
procedure Mul64(var X: Integer64; N: LongWord);
procedure Multiply32x32to64(N1, N2: LongWord; var X: Integer64);
procedure Shr64(var X: Integer64; Count: LongWord);
function StrToInteger64(const S: String; var X: Integer64): Boolean;

implementation

uses
  SysUtils, DelphiCompat;

function Compare64(const N1, N2: Integer64): Integer;
{ If N1 = N2, returns 0.
  If N1 > N2, returns 1.
  If N1 < N2, returns -1. }
begin
  if UInt64(N1) < UInt64(N2) then
    Result := -1
  else if UInt64(N1) > UInt64(N2) then
    Result := 1
  else
    Result := 0;
end;

procedure Dec64(var X: Integer64; N: LongWord);
begin
  Dec(UInt64(X), N);
end;

procedure Dec6464(var X: Integer64; const N: Integer64);
begin
  Dec(UInt64(X), UInt64(N));
end;

procedure Inc64(var X: Integer64; N: LongWord);
begin
  Inc(UInt64(X), N);
end;

procedure Inc6464(var X: Integer64; const N: Integer64);
begin
  Inc(UInt64(X), UInt64(N));
end;

procedure Multiply32x32to64(N1, N2: LongWord; var X: Integer64);
{ Multiplies two 32-bit unsigned integers together and places the result
  in X. }
begin
  UInt64(X) := UInt64(N1) * UInt64(N2);
end;

procedure Mul64(var X: Integer64; N: LongWord);
{ Multiplies X by N, and overwrites X with the result. }
begin
  UInt64(X) := UInt64(X) * UInt64(N);
end;

function Div64(var X: Integer64; const Divisor: LongWord): LongWord;
{ Divides X by Divisor, and overwrites X with the quotient. Returns the
  remainder. }
begin
  Result := UInt64(X) mod UInt64(Divisor);
  UInt64(X) := UInt64(X) div UInt64(Divisor);
end;

function Mod64(const X: Integer64; const Divisor: LongWord): LongWord;
{ Divides X by Divisor and returns the remainder. Unlike Div64, X is left
  intact. }
begin
  Result := UInt64(X) mod UInt64(Divisor);
end;

procedure Shr64(var X: Integer64; Count: Cardinal);
begin
  UInt64(X) := UInt64(X) shr Count;
end;

function StrToInteger64(const S: String; var X: Integer64): Boolean;
{ Converts a string containing a decimal number into an Integer64. Returns
  True if successful. }
var
  I: Integer;
begin
  UInt64(X) := 0;
  Result := False;
  if S <> '' then begin
    for I := 1 to Length(S) do begin
      if not CharInSet(S[I], ['0'..'9']) then
        Exit;
      try
        UInt64(X) := UInt64(X) * 10 + UInt64(Ord(S[I])) - UInt64(Ord('0'));
      except
        on EMathError do  { overflow? }
          Exit;
      end;
    end;
    Result := True;
  end;
end;

function Integer64ToStr(X: Integer64): String;
var
  I: Integer;
  Buf: array[0..31] of Char;  { need at least 20 characters }
begin
  I := High(Buf) + 1;
  repeat
    Dec(I);
    Buf[I] := Chr(Ord('0') + Div64(X, 10));
  until (UInt64(X) = 0);
  SetString(Result, PChar(@Buf[I]), (High(Buf) + 1) - I);
end;

end.
